﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace JednorozmernePoleWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Datové složky třídy (globální proměnné)
        // Specifikace konstanty a inicializace - maximální délka pole
        const int Max = 10;
        // Aktuální rozměr pole - počet prvků - datová složka třídy
        private int pocetPrvku;
        // Přípustná minimální hodnota (dolní mez) prvku - inicializace
        private int minHodnota = 1;
        // Přípustná maximální hodnota (horní mez) prvku - inicializace
        private int maxHodnota = 100;

        //Deklarace pole
        private int[] pole;

        #endregion Datové složky třídy

        #region Pomocné metody
        #region Plnění pole
        private void NahodnePole()
        {
            // Vytvoření instance generátoru náhodného čísla
            Random rnd = new Random();
            // Cyklus s pevným krokem - pro procházení prvků
            for (int i = 0; i < this.pocetPrvku; i++)
            {
                // Přiřazení náhodného čísla v rozmezí intervalu u prvku na pozici [i]
                this.pole[i] = rnd.Next(this.minHodnota, this.maxHodnota);
            }    
        }

        private void NulovatPole()
        {
            for (int i = 0; i < this.pocetPrvku; i++)
            {
                // Přiřazení nuly prvku na pozici [i]
                this.pole[i] = 0;
            }
        }
        #endregion Plnění pole

        #region Zobrazení pole
        private void ZobrazitPoleJakoText()
        {
            string vystup = "";
            // Cyklus s pevným krokem - pro procházení prvků
            for (int i = 0; i < this.pocetPrvku; i++)
            {
                // Přiřazení nuly prvku na pozici [i]
                vystup += this.pole[i].ToString() + ",";
            }
            // Výpis vektoru v objektu typu TextBox na formuláři
            this.TxtVysledek.Text = vystup;
        }

        private void ZobrazitPoleVListView()
        {           
            // Vymazání předchozí textu - přiřazení prázdné hodnoty zdroji objektu
            this.LstVektor.ItemsSource = null;
            // Zobrazení vektoru v objektu na formuláři
            this.LstVektor.ItemsSource = pole;

        }


        #endregion Zobrazení pole

        #region Operace s polem
        private int MinimumVPoli(out int indexPrvku)
        {
            //minimum - hodnota minima
            int minimum;
            // Přiřazení hodnoty počátečního prvku vektoru do minima
            minimum = pole[0];
            // Přiřazení hodnoty 0 do indexu prvku minima (kdyby byl počáteční prvek minimem)
            indexPrvku = 0;
            for (int i = 1; i < this.pocetPrvku; i++)
            {
                if (minimum > pole[i])
                {
                    /* Pokud je nalezen prvek ve vektoru, jehož hodnota je menší 
                     * než minimum... */
                    minimum = pole[i];
                    // Přiřazení indexu aktuálního prvku do indexu minima
                    indexPrvku = i;
                }
            }
            return minimum;
        }

        private int MaximumVPoli(out int indexPrvku)
        {
            //maximum - hodnota minima
            int maximum;
            // Přiřazení hodnoty počátečního prvku vektoru do maxima
            maximum = pole[0];
            // Přiřazení hodnoty 0 do indexu prvku maxima (kdyby byl počáteční prvek maximem)
            indexPrvku = 0;
            for (int i = 1; i < this.pocetPrvku; i++)
            {
                if (maximum < pole[i])
                {
                    /* Pokud je nalezen prvek ve vektoru, jehož hodnota je větší 
                     * než maximum... */
                    maximum = pole[i];
                    // Přiřazení indexu aktuálního prvku do indexu maxima
                    indexPrvku = i;
                }
            }
            return maximum;
        }

        private double PrumerPole()
        {
            //suma - suma prvků vektoru 
            int suma = 0;
            // Cyklus s pevným krokem - pro procházení prvků
            for (int i = 0; i < pocetPrvku; i++)
            {
                // Přičtení dalšího prvku vektoru do sumy
                suma += pole[i];
            }
            return (double)suma/pocetPrvku;
        }

        private void SeraditPole()
        {
            /* Lokální proměnné: i - index ;  pom - pomocná proměnná pro prohození 
             * dvou prvků vektoru; b - booleanovská proměnná - informace, zda
             * došlo k prohození dvou sousedních prvků */
            int pom;
            bool b;
            // Opakuj tak dlouho, dokud bude docházet k prohození prvků vektoru
            do
            {
                // Nedošlo k prohození prvků vektoru
                b = false;
                // Cyklus s pevným krokem - pro procházení od počátečního do předposledního prvku
                for (int i = 0; i < pocetPrvku - 1; i++)
                {
                    /* Pokud je hodnota sousedícího prvku větší než hodnota aktuálně
                     * testovaného prvku, pak... */
                    if (pole[i] > pole[i + 1])
                    {
                        // Ulož hodnotu aktuálního prvku do pomocné proměnné
                        pom = pole[i];
                        // Ulož hodnotu sousedícího prvku do aktuálního prvku
                        pole[i] = pole[i + 1];
                        // Ulož hodnotu z pomocné proměnné do sousedícího prvku
                        pole[i + 1] = pom;
                        // Došlo k prohození prvků
                        b = true;
                    }
                }
            }
            while (b == true);

        }

        #endregion Operace s polem

        #endregion Pomocné metody

        public MainWindow()
        {
            InitializeComponent();
            /* Přiřazení maximálního rozměru vektoru do datové složky určující
             * počet prvků vektoru */
            this.pocetPrvku = Max;

            // Vytvoření instance (objektu) pole s definovanou délkou
            this.pole = new int[this.pocetPrvku];

            /* Přiřazení počtu prvků vektoru do objektu na formuláři určujícího
             * velikost vektoru*/
            this.TxtPocetPrvku.Text = this.pocetPrvku.ToString();

            /* Přiřazení přípustné minimální hodnoty prvku ve vektoru do objektu 
             * na formuláři určujícího mezní hodnotu */
            this.TxtMinHodnota.Text = this.minHodnota.ToString();

            /* přiřazení přípustné maximální hodnoty prvku ve vektoru do objektu (pole) 
             * na formuláři určujícího mezní hodnotu */
            this.TxtMaxHodnota.Text = this.maxHodnota.ToString();
        }

        #region Události
        private void TxtPocetPrvku_LostFocus(object sender, RoutedEventArgs e)
        {
            // Lokální proměnná pro uložení nového počtu prvků vektoru
            if (int.TryParse(this.TxtPocetPrvku.Text, out int novaHodnota) == false)
            {
                MessageBox.Show("Zadal jste špatně počet prvků vektoru!");
                //this.TxtPocetPrvku.Focus();
                return;
            }

            /* Pokud počet prvků je větší než maximální rozměr vektoru, nebo počet
             * prvků vektoru je menší než 1, pak... */
            if ((novaHodnota > Max) || (novaHodnota < 1))
            {
                //MessageBox.Show("Chyba velikosti vektoru, nastavení na max. rozměr: {0}", Max.ToString());
                MessageBox.Show($"Chyba velikosti vektoru, nastavení na max. rozměr: {Max}");

                /* Přiřazení maximálního rozměru vektoru do lokální proměnné určující
                 * počet prvků vektoru */
                novaHodnota = Max;

                /* Přiřazení maximálního rozměru vektoru do objektu na formuláři
                 * určující počet prvků vektoru */
                this.TxtPocetPrvku.Text = novaHodnota.ToString();

            }

            // Pokud nový rozměr vektoru není stejný jako stávající počet prvků vektoru, pak...
            if (novaHodnota != this.pocetPrvku)
            {
                // Uložení nového rozměru pole do datové složky třídy
                this.pocetPrvku = novaHodnota;
                // Vytvoření nové instance pole s novým rozměrem
                this.pole = new int[this.pocetPrvku];
                // Zavolání metody pro zobrazení vektoru do objektu ListView na formuláři

            }

        }

        private void TxtPrvekIndex_LostFocus(object sender, RoutedEventArgs e)
        {
            // Lokální proměnná: index - index prvku
            int index;
            // Uložení indexu prvku do lokální proměnné (konverze textu na celé číslo)
            int.TryParse(this.TxtPrvekIndex.Text, out index);
            /* Pokud index prvku je větší než počet prvků vektoru - 1, nebo index prvku je
             * menší než 0, pak... */
            if ((index > this.pocetPrvku - 1) || (index < 0))
            {
                // Zobrazení dialogového okna s chybou a nastavení indexu na rozměr vektoru - 1
                MessageBox.Show($"Chybný index prvku! Index je nastaven horní mez: {this.pocetPrvku}");
                /* Přiřazení horní meze indexu do objektu na formuláři určujícího index prvku vektoru */
                this.TxtPrvekIndex.Text = string.Format("{0}", this.pocetPrvku - 1);
                //this.TxtPrvekIndex.Text = Convert.ToString(this.pocetPrvku - 1);
            }
        }
        private void TxtHodnota_LostFocus(object sender, RoutedEventArgs e)
        {
            // Lokální proměnná: t - hodnota prvku
            int t;
            // Uložení indexu prvku do lokální proměnné (konverze textu na celé číslo)
            int.TryParse(this.TxtHodnota.Text, out t);
            /* Pokud hodnota prvku je větší než horní mez, nebo hodnota prvku je
             * menší než dolní mez intervalu hodnot ve vektoru, pak... */
            if ((t > this.maxHodnota) || (t < this.minHodnota))
            {
                // Vymázání obsahu objektu na formuláři s hodnotou prvku 
                MessageBox.Show($"Chybná hodnota prvku, musí být v intervalu {this.minHodnota} : {this.maxHodnota}" );
                // Vymázání obsahu objektu na formuláři s hodnotou prvku 
                this.TxtHodnota.Clear();
            }
        }

        private void BtnNahodne_Click(object sender, RoutedEventArgs e)
        {
            this.NahodnePole();
        }

        private void BtnVektorJakoText_Click(object sender, RoutedEventArgs e)
        {
            this.ZobrazitPoleJakoText();
        }

        private void BtnDoMrizky_Click(object sender, RoutedEventArgs e)
        {
            this.ZobrazitPoleVListView();
        }


        private void BtnMinimum_Click(object sender, RoutedEventArgs e)
        {
            /* Přiřazení výstupu metody pro určení minima do lokální proměnné a výstupní 
             * parametr určující index minima */
            int minimum = this.MinimumVPoli(out int indexMinima);
            /* Zobrazení textu spolu s hodnotou nejmenšího prvku a indexu prvku minima 
             * do objektu (TextBox) na formuláři */
            this.TxtVysledek.Text = $"Minimum = {minimum}, index minima = {indexMinima}"; 
        }
        private void BtnMaximum_Click(object sender, RoutedEventArgs e)
        {
            /* Přiřazení výstupu metody pro určení maxima do lokální proměnné a výstupní 
             * parametr určující index maxima */
            int maximum = this.MaximumVPoli(out int indexMaxima);
            /* Zobrazení textu spolu s hodnotou největšího prvku a indexu prvku maxima 
             * do objektu (TextBox) na formuláři */
            this.TxtVysledek.Text = $"Minimum = {maximum}, index maxima = {indexMaxima}";
        }

        private void BtnPrumer_Click(object sender, RoutedEventArgs e)
        {
            // prumer - průměr prvků v poli
            double prumer = this.PrumerPole();
             // Zobrazení textu 'Průměr = ' spolu s vypočteným průměrem
            this.TxtVysledek.Text = $"Průměr pole = {prumer}";
        }

        private void BtnSeradit_Click(object sender, RoutedEventArgs e)
        {
            // Zavolání metody pro vzestupné třídění prvků v poli
            this.SeraditPole();
            // Zavolání metody pro zobrazení vektoru do objektu (TextBox) na formuláři
            this.ZobrazitPoleJakoText();
        }

        #endregion Události


    }
}
